unit Main;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, ExtCtrls, ComCtrls, Menus, StdCtrls;

type
  TFrmMain = class(TForm)
    Shape: TShape;
    Container_haut: TBevel;
    Menu: TMainMenu;
    Jeux: TMenuItem;
    NouvellePartie: TMenuItem;
    Separateur1: TMenuItem;
    Joueurs: TMenuItem;
    Separateur2: TMenuItem;
    Fermer: TMenuItem;
    lblJetonsRouge: TLabel;
    lblJoueurJaune: TLabel;
    lblJetonsJaune: TLabel;
    EffacerScores: TMenuItem;
    lblJoueurRouge: TLabel;
    lblNbJetonsRouge: TLabel;
    lblNbJetonsJaune: TLabel;
    lblGagneRouge: TLabel;
    lblGagneJaune: TLabel;
    lblNbGagneRouge: TLabel;
    lblNbGagneJaune: TLabel;
    APropos: TMenuItem;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FermerClick(Sender: TObject);
    procedure NouvellePartieClick(Sender: TObject);
    procedure JoueursClick(Sender: TObject);
    procedure SharpeOnMouseDown(Sender: TObject;
                Button:TMouseButton; Shift:TShiftState; A,B:Integer);
    procedure JetonOnMouseMove(Sender: TObject; Shift: TShiftState; X,
                Y: Integer);
    procedure GrilleOnMouseMove(Sender: TObject; Shift: TShiftState; X,
                Y: Integer);
    procedure EffacerScoresClick(Sender: TObject);
    procedure AProposClick(Sender: TObject);
    procedure FormKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);

  private
    { Dclarations prives }
    procedure DeplacementJetonClavier(Color:TColor; KeyGauche, KeyValide, KeyDroite, Key: Integer);
    procedure CreateJeton(composant: TWinControl; curcolor:TColor;
                xpos, ypos, Position:byte);
    procedure DestroyJeton;
    procedure InitialiseJeton(color:TColor; Ancienne, Nouvelle:byte);
    procedure DeplacementJetonSouris(color:TColor; droite:boolean);
    procedure AfficheJeton(color:TColor; Position:Byte);
    procedure DescenteJeton(posx, nb:integer; color:TColor);
    procedure SetNbJetonsRouge(NbJeton:byte);
    procedure SetNbJetonsJaune(NbJeton:byte);
    procedure SetScoreRouge(Score:integer);
    procedure SetScoreJaune(Score:integer);
  public
    { Dclarations publiques }
    procedure SetDeplacementSourisRouge(Valeur : boolean);
    procedure SetDeplacementSourisJaune(Valeur : Boolean);
    procedure SetToucheRouge(Gauche,Valider,Droite : Integer);
    procedure SetToucheJaune(Gauche,Valider,Droite : Integer);
    procedure SetJoueurRouge(NomJoueur : string);
    procedure SetJoueurJaune(NomJoueur : string);
    procedure CreatePlateau(composant: TWinControl; xpos, ypos: byte);
    procedure DestroyPlateau;
    procedure InitialisePlateau;
    function LigneJeton(xpos:byte):integer;
    function GetCouleurJeton(posx, posy:byte):TColor;
    function TestVictoire(posx, posy : byte; curcolor:TColor):boolean;
    procedure SauvegardeAlignement(posx, posy, indice:byte);
    procedure ChangementCouleurLigne(curcolor:TColor);
    function remplir_Plateau(posx:byte; color:TColor) : TColor;
  end;

type TAlignement = record
  posx, posy : byte;
end;

const
  NBLIGNE = 6;
  NBCOLONNE = 7;
  ESPACEMENT = 6;
  TAILLEJETON = 65;
  COULEURDEFAUT = clskyblue;
  NBJETONS = 21;
var
  FrmMain: TFrmMain;
  CurColor:TColor;
  DeplacementSourisRouge, DeplacementSourisJaune : boolean;
  JKeyGauche, JKeyValide, JKeyDroite : Integer;
  RKeyGauche, RKeyValide, RKeyDroite : Integer;
  ScoreRouge, ScoreJaune : integer;
  Jeton : array [0..NBCOLONNE] of tshape;
  PositionJeton : byte;
  Grille:ARRAY [0..NBCOLONNE,0..NBLIGNE] OF TShape;
  Alignement: array [0..4] of TAlignement;
  Arret: Boolean;
  JoueurRouge,JoueurJaune:String;
  JetonsRouge, JetonsJaune : byte;

implementation
uses
  UVerifVictoire,UJoueurs, ABOUT;
{$R *.dfm}

{$REGION ' action provenant du formulaire '}
{------------------------------------------------------------------------------
  Procdure pour fermer l'application
-------------------------------------------------------------------------------}
procedure TFrmMain.FermerClick(Sender: TObject);
begin
  if MessageDlg('Voulez vous quitter l''application ?',mtConfirmation,[Mbyes,MbNo],0)=MrYes then
    Application.Terminate;
end;

{------------------------------------------------------------------------------
  Procdure de destruction des composants
-------------------------------------------------------------------------------}
procedure TFrmMain.FormDestroy(Sender: TObject);
begin
  DestroyJeton;
  DestroyPlateau;
end;

{------------------------------------------------------------------------------
  Procdure de cration du formulaire et d'initialisation
-------------------------------------------------------------------------------}
procedure TFrmMain.FormCreate(Sender: TObject);
begin
  // couleur par defaut
  curcolor := clred;
  // initialise la variable arret
  Arret := false;
  // Deplacement du jetons avec la souris
  DeplacementSourisRouge := true;
  DeplacementSourisJaune := true;
  // met les scores  0
  self.SetScoreRouge(0);
  self.SetScoreJaune(0);
  // initialise la position du jeton
  PositionJeton := 3;
  // initialise le nombre de jetons restants
  self.SetNbJetonsRouge(NBJETONS);
  self.SetNbJetonsJaune(NBJETONS);
  // touche par dfaut
  RKeyGauche := -1;
  RKeyValide := -1;
  RKeyDroite := -1;
  JKeyGauche := -1;
  JKeyValide := -1;
  JKeyDroite := -1;
  // initialise le nom des joueurs
  JoueurRouge := 'Joueur1';
  JoueurJaune := 'Joueur2';
  // affichage des informations
  self.lblJoueurRouge.Caption := JoueurRouge;
  self.lblJoueurJaune.Caption := JoueurJaune;
  // Cration du tableau du haut et du plateau
  CreateJeton(FrmMain, curcolor, 8, 8, PositionJeton);
  CreatePlateau(FrmMain, 8,100);
  // viter le scintillement des jetons
  DoubleBuffered := true;
end;

{------------------------------------------------------------------------------
  Procdure de gestion des touches
-------------------------------------------------------------------------------}
procedure TFrmMain.FormKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  // Verifie que la partie n'est pas finie
  if not Arret then
  begin
    // verifie que le joueur rouge a choisit le deplacement par clavier
    if (CurColor = clred) and (DeplacementSourisRouge=false) then
      // dplacement du jeton
      DeplacementJetonClavier(CurColor, RKeyGauche, RKeyValide, RKeyDroite, key)
    // verifie que le joueur jaune a choisit le deplacement par clavier
    else if (curcolor = clyellow) and (DeplacementSourisJaune=false) then
      // dplacemnt du jeton
      DeplacementJetonClavier(CurColor, JKeyGauche, JKeyValide, JKeyDroite, key);
  end;
end;

{------------------------------------------------------------------------------
  Procdure provenant du menu
-------------------------------------------------------------------------------}
procedure TFrmMain.JoueursClick(Sender: TObject);
var
  RenseignementJoueurs : TFrmJoueurs;
begin
  // ouverture du formulaire joueur
  RenseignementJoueurs := TFrmJoueurs.CreateFrm(self,JoueurRouge,JoueurJaune,DeplacementSourisRouge, DeplacementSourisJaune,RKeyGauche, RKeyValide, RKeyDroite,JKeyGauche, JKeyValide, JKeyDroite);
  RenseignementJoueurs.Show;
end;

procedure TFrmMain.EffacerScoresClick(Sender: TObject);
begin
  // remet  0 les scores
  self.SetScoreRouge(0);
  self.SetScoreJaune(0);
end;

procedure TFrmMain.AProposClick(Sender: TObject);
var
  ap : TAboutBox;
begin
  // ouverture du formulaire "A Propos"
  ap := TAboutBox.Create(self);
  ap.Show;
end;

{------------------------------------------------------------------------------
  Procdure pour initialiser une nouvelle partie
-------------------------------------------------------------------------------}
procedure TFrmMain.NouvellePartieClick(Sender: TObject);
begin
  // initialise la variable
  Arret := false;
  // affiche tous les jetons du plateau en blanc
  InitialisePlateau;
end;

{------------------------------------------------------------------------------
  Gestion de la souris
-------------------------------------------------------------------------------}
procedure TFrmMain.JetonOnMouseMove(Sender: TObject; Shift: TShiftState; X,
                Y: Integer);
var
  i : byte;
begin
  // verifie que la partie n'est pas finie et
  // que le joueur a choisit le deplacement de la souris
  if not Arret and (((curcolor = clred) and DeplacementSourisRouge) or
  ((curcolor = clyellow) and DeplacementSourisJaune)) then
  begin
    // parcours le tableau  la recherche de la colonne selectionn
    for i := 0 to NbColonne - 1 do
    if (sender = Jeton[i]) and (PositionJeton <> i) then
    begin
      // affiche le jeton dans la barre du haut
      InitialiseJeton(curcolor, positionJeton, i);
      positionJeton := i;
    end;
  end;
end;

procedure TFrmMain.SharpeOnMouseDown(Sender: TObject;
                Button:TMouseButton; Shift:TShiftState; A,B:Integer);
begin
  // verifie que la partie n'est pas finie et
  // que le joueur a choisit le deplacement de la souris
  if not Arret and (((curcolor = clred) and DeplacementSourisRouge) or
  ((curcolor = clyellow) and DeplacementSourisJaune)) then
  begin
    // affiche le jeton dans la grille si la ligne est remplie
    curcolor := remplir_Plateau(PositionJeton, curColor);
    // change la couleur du jeton du haut
    Jeton[PositionJeton].Brush.Color := curcolor;
  end;
end;

procedure TFrmMain.GrilleOnMouseMove(Sender: TObject; Shift: TShiftState; X,
                Y: Integer);
var
  posx,posy : byte;
begin
  // verifie que la partie n'est pas finie et
  // que le joueur a choisit le deplacement de la souris
  if not Arret and (((curcolor = clred) and DeplacementSourisRouge) or
  ((curcolor = clyellow) and DeplacementSourisJaune)) then
  begin
    // parcours le tableau  la recherche de la colonne selectionn
    for posy := 0 to NBLIGNE - 1 do
      for posx := 0 to NbColonne - 1 do
        if (sender = Grille[posx][posy]) and (PositionJeton <> posx) then
        begin
          // affiche le jeton dans la barre du haut
          InitialiseJeton(curcolor, positionJeton, posx);
          positionJeton := posx;
        end;
  end;
end;

{$ENDREGION}

{$REGION ' enregistrement provenant du formulaire UJoueurs '}
{------------------------------------------------------------------------------
  Changement des informations concernant des informations sur les joueurs
-------------------------------------------------------------------------------}
procedure TFrmMain.SetDeplacementSourisRouge(Valeur : boolean);
begin
  DeplacementSourisRouge := Valeur;
end;

procedure TFrmMain.SetDeplacementSourisJaune(Valeur : Boolean);
begin
  DeplacementSourisJaune := Valeur;
end;

procedure TFrmMain.SetToucheRouge(Gauche,Valider,Droite : Integer);
begin
  RKeyGauche := Gauche;
  RKeyValide := Valider;
  RKeyDroite := Droite;
end;

procedure TFrmMain.SetToucheJaune(Gauche,Valider,Droite : Integer);
  begin
    JKeyGauche := Gauche;
    JKeyValide := Valider;
    JKeyDroite := Droite;
  end;

procedure TFrmMain.SetJoueurRouge(NomJoueur : string);
begin
  JoueurRouge := NomJoueur;
  self.lblJoueurRouge.Caption := JoueurRouge;
end;

procedure TFrmMain.SetJoueurJaune(NomJoueur : string);
begin
  JoueurJaune := NomJoueur;
  self.lblJoueurJaune.Caption := JoueurJaune;
end;
{$ENDREGION}

{$REGION ' Dplacement du jeton dans la barre du haut '}
{------------------------------------------------------------------------------
  Cration du TShape Jeton
-------------------------------------------------------------------------------}
procedure TFrmMain.CreateJeton(composant: TWinControl; curcolor:TColor;
                xpos, ypos, Position:byte);
var
  x : byte;
begin
  for x := 0 to NbColonne - 1 do
  begin
    Jeton[x]              := tshape.Create(composant);
    Jeton[x].Left         := (TAILLEJETON*x)+(ESPACEMENT*x)+xpos;
    Jeton[x].Top          := ypos;
    Jeton[x].Width        := TAILLEJETON;
    Jeton[x].Height       := TAILLEJETON;
    Jeton[x].Parent       := composant;
    Jeton[x].Shape        := stcircle;
    Jeton[x].Brush.Style  := BsClear;
    Jeton[x].Pen.Style    := PsClear;
    Jeton[x].OnMouseMove  := JetonOnMouseMove;
    Jeton[x].OnMouseDown  := SharpeOnMouseDown;
  end;
  AfficheJeton(curcolor, Position);
end;

{------------------------------------------------------------------------------
  Destruction du TShape Jeton
-------------------------------------------------------------------------------}
procedure TFrmMain.DestroyJeton;
var
  x : byte;
begin
  for x := 0 to NbColonne - 1 do
    Jeton[x].Destroy;
end;

{------------------------------------------------------------------------------
  Initialise le TShape Jeton
-------------------------------------------------------------------------------}
procedure TFrmMain.InitialiseJeton(color:TColor; Ancienne, Nouvelle:byte);
begin
  Jeton[Ancienne].Brush.Style := BsClear;
  Jeton[Ancienne].Pen.Style   := PsClear;
  AfficheJeton(color, Nouvelle);
end;

{------------------------------------------------------------------------------
  Affichage du jeton dans le TShape Jeton
-------------------------------------------------------------------------------}
procedure TFrmMain.AfficheJeton(color:TColor; Position:Byte);
begin
  Jeton[Position].Brush.Style   := BsSolid;
  Jeton[Position].Pen.Style     := PsSolid;
  Jeton[Position].Brush.Color   := color;
end;

{------------------------------------------------------------------------------
  Dplacement du jeton dans le TShape Jeton
-------------------------------------------------------------------------------}
procedure TFrmMain.DeplacementJetonSouris(color:TColor; droite:boolean);
begin
  // verifie que le jeton est autoris  se deplacer
  if (not(PositionJeton = 0) and not droite) or
  (not (PositionJeton = NbColonne-1) and droite) then
  begin
    if droite then
    begin
      // deplacement du jeton vers la droite
      initialiseJeton(color, PositionJeton, PositionJeton +1);
      PositionJeton := PositionJeton +1;
    end
    else
    begin
      // deplacement du jeton vers la gauche
      initialiseJeton(color, PositionJeton, PositionJeton -1);
      PositionJeton := PositionJeton -1;
    end;
  end;
end;

{------------------------------------------------------------------------------
  Procdure pour permettre le deplacement de jeton avec le clavier
-------------------------------------------------------------------------------}
procedure TFrmMain.DeplacementJetonClavier(color:TColor; KeyGauche, KeyValide, KeyDroite, Key: Integer);
begin
   if (KeyDroite = Key) then DeplacementJetonSouris(color, true)
   else if (KeyGauche=Key) then DeplacementJetonSouris(color, false)
   else if (KeyValide=Key) then
   begin
     // affiche le jetons sur le plateau si la ligne n'est pas remplie
     curcolor := remplir_Plateau(PositionJeton, curcolor);
     // change la couleur du jeton du haut
     Jeton[PositionJeton].Brush.Color := curcolor;
   end;
end;
{$ENDREGION}

{$REGION ' Jetons dans le Plateau '}
{------------------------------------------------------------------------------
  Cration du TShape Plateau
-------------------------------------------------------------------------------}
procedure TFrmMain.CreatePlateau(composant: TWinControl; xpos, ypos: byte);
var x,y : byte;
begin
  FOR y:=0 TO nbLigne-1 DO
  FOR x:=0 TO nbColonne-1 DO
    BEGIN
      Grille[x,y]               := Tshape.Create(composant);
      Grille[x,y].Left          := (TAILLEJETON*x)+(ESPACEMENT*x)+xpos;
      Grille[x,y].Top           := (TAILLEJETON*y)+(ESPACEMENT*y)+ypos;
      Grille[x,y].Width         := TAILLEJETON;
      Grille[x,y].Height        := TAILLEJETON;
      Grille[x,y].Parent        := composant;
      Grille[x,y].Shape         := stCircle;
      Grille[x,y].Brush.Color   := COULEURDEFAUT;
      Grille[x,y].OnMouseMove   := GrilleOnMouseMove;
      Grille[x,y].OnMouseDown   := SharpeOnMouseDown;
    END;
end;

{------------------------------------------------------------------------------
  Destruction du TShape Plateau
-------------------------------------------------------------------------------}
procedure TFrmMain.DestroyPlateau;
var x,y : byte;
begin
  FOR y:=0 TO NbLigne - 1 DO
  FOR x:=0 TO NbColonne - 1 DO
    Grille[x,y].Destroy;
end;

{------------------------------------------------------------------------------
  Initialise le TShape Plateau
-------------------------------------------------------------------------------}
procedure TFrmMain.InitialisePlateau;
var x,y : byte;
begin
  for y := 0 to NbLigne - 1 do
  for x := 0 to NbColonne - 1 do
    Grille[x,y].Brush.Color := COULEURDEFAUT;
  // affiche le nombre de jeton restant
  self.SetNbJetonsRouge(NBJETONS);
  self.SetNbJetonsJaune(NBJETONS);
end;

{------------------------------------------------------------------------------
  Procedure de gestion de la descente du jeton
-------------------------------------------------------------------------------}
procedure TFrmMain.DescenteJeton(posx, nb:integer; color:TColor);
var
  y : integer;
begin
  // parcourt la procedure jusqu' la position du dernier jeton
  for y := 0 to nb - 1 do
  begin
    Grille[posx, y].Brush.Color := color;
    Grille[posx,y].Refresh;
    // pause de 150 ms
    sleep(150);
    Grille[posx, y].Brush.Color := COULEURDEFAUT;
  end;
  Grille[posx, nb].Brush.Color := color;
end;

{------------------------------------------------------------------------------
  Fonction qui permet de rajouter un jeton sur le TShape Plateau
-------------------------------------------------------------------------------}
function TFrmMain.remplir_Plateau(posx:byte; color:TColor) : TColor;
VAR
  posy : integer;
BEGIN
  result := color;
  if not arret then
  begin
    // recupere la position du jeton qui doit changer de couleur
    posy := LigneJeton(posx);
    if posy <> -1 then
    begin
      // affiche le jeton qui descent
      DescenteJeton(posx, posy, color);
      // affichage du nombre de jetons restants
      if color = clred then
        self.SetNbJetonsRouge(JetonsRouge -1)
      else
        self.SetNbJetonsJaune(JetonsJaune -1);
      // test si le joueur a gagn si aux moins 4 jetons ont t jou
      if ((JetonsJaune < 18) or (JetonsRouge < 18)) and TestVictoire(posx, posy, color) then
      begin
        // met la ligne en vert
        ChangementCouleurLigne(clgreen);
        if color = clRed then
        begin
          MessageDlg('Victoire de ' + JoueurRouge , mtInformation,[MbOk],0);
          self.SetScoreRouge(ScoreRouge + 1);
        end
        else
        begin
          MessageDlg('Victoire de ' + JoueurJaune , mtInformation,[MbOk],0);
          self.SetScoreJaune(ScoreJaune + 1);
        end;
        // arret de la partie en cours
        Arret := true;
      end
      // verifie si tout les jetons ont t jou
      else if (JetonsRouge = 0) and (JetonsJaune = 0) then
      begin
        MessageDlg('Match nul', mtInformation,[MbOk],0);
        Arret := true;
      end;
      // change de joueur et la couleur du jeton
      IF(Color=clRed)THEN
        result:=clYellow
      ELSE
        result:=clRed;
    end;
  end;
END;

{------------------------------------------------------------------------------
  Procedure qui permet le changement de couleur de la ligne
-------------------------------------------------------------------------------}
procedure TFrmMain.ChangementCouleurLigne(curcolor:TColor);
var i:byte;
begin
  for i := 0 to 3 do
    Grille[Alignement[i].posx,Alignement[i].posy].Brush.Color:=CurColor;
end;

{------------------------------------------------------------------------------
  Procedure qui sauvegarde la position des jetons qui compose l'alignement
-------------------------------------------------------------------------------}
procedure TFrmMain.SauvegardeAlignement(posx, posy, indice:byte);
begin
  Alignement[indice-1].posx := posx;
  Alignement[indice-1].posy := posy;
end;

{------------------------------------------------------------------------------
  Test si un alignement de 4 jetons est present
-------------------------------------------------------------------------------}
function TFrmMain.TestVictoire(posx, posy: byte; curcolor:TColor):Boolean;
var resultat: boolean;
begin
  // enregistre le jeton qui vient d'tre jou
  frmmain.SauvegardeAlignement(posx, posy, 1);
  resultat := UVerifVictoire.TestHorizontal(posx,posy,curcolor, NbColonne);
  if not resultat then
    resultat := UVerifVictoire.TestVertical(posx, posy, curcolor, NbLigne);
  if not resultat then
    resultat := UVerifVictoire.TestDiag1(posx, posy, curcolor, NbLigne, Nbcolonne);
  if not resultat then
    resultat := UVerifVictoire.TestDiag2(posx, posy, curcolor, NbLigne, NbColonne);
  result := resultat;
end;

{------------------------------------------------------------------------------
  Renvoie la position du dernier jeton
-------------------------------------------------------------------------------}
function TFrmMain.LigneJeton(xpos:byte):integer;
var
  y:byte;
begin
  y := NbLigne;
  result := -1;
  repeat
    y := y - 1;
    if (Grille[xpos,y].Brush.Color = COULEURDEFAUT) then
      result := y;
  until (y = 0) or (result <> -1);
end;

{------------------------------------------------------------------------------
  Renvoie la couleur du Jeton
-------------------------------------------------------------------------------}
function TFrmMain.GetCouleurJeton(posx, posy:byte):TColor;
begin
  result := Grille[posx, posy].Brush.Color;
end;

{------------------------------------------------------------------------------
  Enregistre et affiche le nombre de jetons rouge restant
-------------------------------------------------------------------------------}
procedure TFrmMain.SetNbJetonsRouge(NbJeton:byte);
begin
  JetonsRouge := NbJeton;
  self.lblNbJetonsRouge.Caption := inttostr(JetonsRouge);
end;

{------------------------------------------------------------------------------
  Enregistre et affiche le nombre de jetons jaune restant
-------------------------------------------------------------------------------}
procedure TFrmMain.SetNbJetonsJaune(NbJeton:byte);
begin
  JetonsJaune := NbJeton;
  self.lblNbJetonsJaune.Caption := inttostr(JetonsJaune);
end;

{------------------------------------------------------------------------------
  Enregistre et affiche le score du joueur rouge
-------------------------------------------------------------------------------}
procedure TFrmMain.SetScoreRouge(Score:integer);
begin
  ScoreRouge := Score;
  self.lblNbGagneRouge.Caption := inttostr(ScoreRouge);
end;

{------------------------------------------------------------------------------
  Enregistre et affiche le score du joueur jaune
-------------------------------------------------------------------------------}
procedure TFrmMain.SetScoreJaune(Score:integer);
begin
  ScoreJaune := Score;
  self.lblNbGagneJaune.Caption := inttostr(ScoreJaune);
end;
{$ENDREGION}
end.
